package data;

import conexoes.ConexaoMysql;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class EnderecoDao {
    private final ConexaoMysql conexao;
    
    public EnderecoDao() throws SQLException, ClassNotFoundException {
        this.conexao = new ConexaoMysql();
    }
    
    public Long inserir(Endereco endereco, long idUsuario) throws SQLException, ClassNotFoundException {
        Long id = null;
        String sqlEndereco = "INSERT INTO enderecos(cep, bairro, cidade, rua, numero, complemento, ativo) VALUES (?, ?, ?, ?, ?, ?, true);";
        String sqlEnderecoUsuario = "INSERT INTO endereco_usuario(endereco_id, usuario_id) VALUES (?, ?);";
        
        try {
            PreparedStatement stmtEndereco = this.conexao.getConnection().prepareStatement(sqlEndereco, Statement.RETURN_GENERATED_KEYS);
            stmtEndereco.setString(1, endereco.getCep());
            stmtEndereco.setString(2, endereco.getBairro());
            stmtEndereco.setString(3, endereco.getCidade());
            stmtEndereco.setString(4, endereco.getRua());
            stmtEndereco.setString(5, endereco.getNumero());
            stmtEndereco.setString(6, endereco.getComplemento());                            
            stmtEndereco.executeUpdate();
                        
            ResultSet rs = stmtEndereco.getGeneratedKeys();         
            if(rs.next()) {
                id = rs.getLong(1);
                
                PreparedStatement stmtEnderecoUsuario = this.conexao.getConnection().prepareStatement(sqlEnderecoUsuario);
                stmtEnderecoUsuario.setLong(1, rs.getLong(1));
                stmtEnderecoUsuario.setLong(2, idUsuario);        
                stmtEnderecoUsuario.executeUpdate();
            }
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        return id;
    }
    
    public int alterar(Endereco e) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "UPDATE enderecos SET cep = ?, bairro = ?, cidade = ?, rua = ?, numero = ?, complemento = ? WHERE id = ?;";                
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setString(1, e.getCep());
            stmt.setString(2, e.getBairro());
            stmt.setString(3, e.getCidade());
            stmt.setString(4, e.getRua());
            stmt.setString(5, e.getNumero());
            stmt.setString(6, e.getComplemento());
            stmt.setLong(7, e.getId());
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException ex) {
            this.conexao.rollback();
            throw ex;
        }
        return alterados;
    }
    
    public int excluir(long id) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "UPDATE enderecos SET ativo = false WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setLong(1, id);
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }
    
    public Endereco obterEndereco(long id) throws SQLException, ClassNotFoundException {                
        String sql = "SELECT e.id, e.cep, e.bairro, e.cidade, e.rua, e.numero, e.complemento" +
            " FROM enderecos as e" +
            " WHERE e.id = ?";
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setLong(1, id);
            
            ResultSet rs = stmt.executeQuery();
            if (rs.next()) {
                return parser(rs);
            }            
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        
        return null;
    }    
    
    public List<Endereco> listar(long idUsuario) throws SQLException {
        String sql = "SELECT e.id, e.cep, e.bairro, e.cidade, e.rua, e.numero, e.complemento" +
            " FROM enderecos as e" +
            " INNER JOIN endereco_usuario eu" +
            " WHERE (eu.usuario_id = ?) AND (eu.endereco_id = e.id) AND (e.ativo = true);";
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);                                    
            stmt.setLong(1, idUsuario);
            ResultSet rs = stmt.executeQuery();
            
            List<Endereco> enderecos = new ArrayList<>();            

            while (rs.next()) {
                enderecos.add(parser(rs));
            }
            
            return enderecos;
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
    }
    
    private Endereco parser(ResultSet rs) throws SQLException {
        Endereco e = new Endereco();
        
        e.setId(rs.getLong("e.id"));
        e.setCep(rs.getString("cep"));
        e.setBairro(rs.getString("bairro"));
        e.setCidade(rs.getString("cidade"));
        e.setRua(rs.getString("rua"));
        e.setNumero(rs.getString("numero"));
        e.setComplemento(rs.getString("complemento"));        
        
        return e;
    }
}
