package data;

import conexoes.ConexaoMysql;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class FornecedorDao {
    private final ConexaoMysql conexao;
    
    public FornecedorDao() throws SQLException, ClassNotFoundException {
        this.conexao = new ConexaoMysql();
    }
    
    public Long inserir(Fornecedor fornecedor) throws SQLException, ClassNotFoundException {
        Long id = null;
        String sql = "INSERT INTO fornecedores (nome, telefone, endereco_id, ativo) VALUES (?, ?, ?, true);";        
        String sqlEndereco = "INSERT INTO enderecos(cep, bairro, cidade, rua, numero, complemento, ativo) VALUES (?, ?, ?, ?, ?, ?, true);";

        try {                        
            PreparedStatement stmtEndereco = this.conexao.getConnection().prepareStatement(sqlEndereco, Statement.RETURN_GENERATED_KEYS);
            stmtEndereco.setString(1, fornecedor.getEndereco().getCep());
            stmtEndereco.setString(2, fornecedor.getEndereco().getBairro());
            stmtEndereco.setString(3, fornecedor.getEndereco().getCidade());
            stmtEndereco.setString(4, fornecedor.getEndereco().getRua());
            stmtEndereco.setString(5, fornecedor.getEndereco().getNumero());
            stmtEndereco.setString(6, fornecedor.getEndereco().getComplemento());

            stmtEndereco.executeUpdate();
            ResultSet rs = stmtEndereco.getGeneratedKeys();
            if(rs.next()) {
                id = rs.getLong(1);
                
                PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
                stmt.setString(1, fornecedor.getNome());
                stmt.setString(2, fornecedor.getTelefone());                                
                stmt.setLong(3, id);
                
                stmt.executeUpdate();
                
                this.conexao.commit();
            }            
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        
        return id;
    }

    public int alterar(Fornecedor fornecedor) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "UPDATE fornecedores SET nome = ?, telefone = ?, ativo = ? WHERE id = ?;";
        String sqlEndereco = "UPDATE enderecos SET cep = ?, bairro = ?, cidade = ?, rua = ?, numero = ?, complemento = ?) WHERE id = ?;";
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setString(1, fornecedor.getNome());
            stmt.setString(2, fornecedor.getTelefone());
            stmt.setBoolean(3, fornecedor.isAtivo());
            stmt.setLong(4, fornecedor.getId());            
            alterados = stmt.executeUpdate();
            
            if (alterados > 0) {
                PreparedStatement stmtEndereco = this.conexao.getConnection().prepareStatement(sqlEndereco);
                stmtEndereco.setString(1, fornecedor.getEndereco().getCep());
                stmtEndereco.setString(2, fornecedor.getEndereco().getBairro());
                stmtEndereco.setString(3, fornecedor.getEndereco().getCidade());
                stmtEndereco.setString(4, fornecedor.getEndereco().getRua());
                stmtEndereco.setString(5, fornecedor.getEndereco().getNumero());
                stmtEndereco.setString(6, fornecedor.getEndereco().getComplemento());
                stmtEndereco.setLong(7, fornecedor.getEndereco().getId());
                
                this.conexao.commit();
            }
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }

    public int excluir(long id) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "DELETE FROM fornecedores WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setLong(1, id);
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }
    
    public Fornecedor obterFornecedor(long id) throws SQLException, ClassNotFoundException {                        
        String sql = "SELECT * FROM fornecedores as f"
            + " INNER JOIN enderecos as e ON f.endereco_id = e.id"
            + " WHERE f.id = ?;";              
        
        try {                                  
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setLong(1, id);
            
            ResultSet rs = stmt.executeQuery();
            if (rs.next()) {
                return parser(rs);
            }            
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        
        return null;
    }
    
    public List<Fornecedor> listar() throws SQLException, ClassNotFoundException {                
        String sql = "SELECT * FROM fornecedores as f"
                + " INNER JOIN enderecos as e ON f.endereco_id = e.id"
                + " WHERE f.ativo = true"
                + " ORDER BY f.id;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);                                    
            ResultSet rs = stmt.executeQuery();
            
            List<Fornecedor> fornecedores = new ArrayList<>();
            
            while (rs.next()) {
                fornecedores.add(parser(rs));
            }            
            
            return fornecedores;
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }                       
    }
    
    private Fornecedor parser(ResultSet rs) throws SQLException {
        Fornecedor f = new Fornecedor();        
        Endereco e = new Endereco();
        
        e.setCep(rs.getString("cep"));
        e.setBairro(rs.getString("bairro"));
        e.setCidade(rs.getString("cidade"));
        e.setRua(rs.getString("rua"));
        e.setNumero(rs.getString("numero"));
        e.setComplemento(rs.getString("complemento"));
        
        f.setId(rs.getLong("id"));
        f.setNome(rs.getString("nome"));
        f.setTelefone(rs.getString("telefone"));
        f.setAtivo(rs.getBoolean("ativo"));
        f.setEndereco(e);
        
        return f;
    }    
}