package data;

import conexoes.ConexaoMysql;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class ItemEstoqueDao {
    private final ConexaoMysql conexao;
    
    public ItemEstoqueDao() throws SQLException, ClassNotFoundException {
        this.conexao = new ConexaoMysql();
    }
    
    public void inserir(ItemEstoque item) throws SQLException, ClassNotFoundException {        
        String sql = "INSERT INTO itens_estoque (produto_id, fornecedor_id, quantidade) VALUES (?, ?, ?);";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setLong(1, item.getProduto().getId());
            stmt.setLong(2, item.getFornecedor().getId());
            stmt.setInt(3, item.getQuantidade());
            stmt.executeUpdate();
            
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }                
    }
    
    public int alterar(ItemEstoque item) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "UPDATE itens_estoque SET quantidade = ? WHERE (produto_id = ?) AND (fornecedor_id = ?);";
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setInt(1, item.getQuantidade());
            stmt.setLong(2, item.getProduto().getId());
            stmt.setLong(3, item.getFornecedor().getId());          
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }
    
    public int excluir(ItemEstoque item) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "DELETE FROM itens_estoque WHERE (produto_id = ?) AND (fornecedor_id = ?);";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);     
            stmt.setLong(1, item.getProduto().getId());
            stmt.setLong(2, item.getFornecedor().getId());
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }
    
    public ItemEstoque obterItem(long idProduto, long idFornecedor) throws SQLException, ClassNotFoundException {                
        String sql = "SELECT * FROM itens_estoque WHERE (produto_id = ?) AND (fornecedor_id = ?);";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setLong(1, idProduto);
            stmt.setLong(2, idFornecedor);
            
            ResultSet rs = stmt.executeQuery();
            if (rs.next()) {
                return parser(rs);
            }            
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        
        return null;
    }
    
    public List<ItemEstoque> listar() throws SQLException, ClassNotFoundException {                
        String sql = "SELECT p.id, p.nome, f.id, f.nome, ie.quantidade FROM itens_estoque as ie" +
            " INNER JOIN produtos as p ON ie.produto_id = p.id" +
            " INNER JOIN fornecedores f ON ie.fornecedor_id = f.id" +
            " ORDER BY p.id;";
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);                                    
            ResultSet rs = stmt.executeQuery();
            
            List<ItemEstoque> itensEstoque = new ArrayList<>();
            
            while (rs.next()) {
                itensEstoque.add(parser(rs));
            }            
            
            this.conexao.close();
            return itensEstoque;
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        } finally {
            this.conexao.close();
        }                       
    }
    
    public void zerarEstoque() throws SQLException {
        String sql = "UPDATE itens_estoque SET quantidade = 0";
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);                 
            stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
    }
    
    private ItemEstoque parser(ResultSet rs) throws SQLException {
        ItemEstoque i = new ItemEstoque();
        Produto p = new Produto();
        Fornecedor f = new Fornecedor();
        
        p.setId(rs.getLong("p.id"));
        p.setNome(rs.getString("p.nome"));
        
        f.setId(rs.getLong("f.id"));
        f.setNome(rs.getString("f.nome"));
        
        i.setProduto(p);
        i.setFornecedor(f);        
        i.setQuantidade(rs.getInt("ie.quantidade"));
        
        return i;
    }
}
