package data;

import conexoes.ConexaoMysql;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class PedidoDao {
    private final ConexaoMysql conexao;
    
    public PedidoDao() throws SQLException, ClassNotFoundException {
        this.conexao = new ConexaoMysql();
    }
    
    public Long inserir(Pedido pedido) throws SQLException, ClassNotFoundException {
        Long id = null;
        String sql = "INSERT INTO pedidos (date_time, usuario_id, endereco_id, situacao) VALUES (NOW(), ?, ?, 'Novo');";        
        String sqlItemPedido = "INSERT INTO item_pedido(pedido_id, produto_id, fornecedor_id, valor_unitario, quantidade) VALUES (?, ?, ?, ?, ?);";        

        try {                        
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
            stmt.setLong(1, pedido.getUsuario().getId());
            stmt.setLong(2, pedido.getEndereco().getId());            
            stmt.executeUpdate();
            
            ResultSet rs = stmt.getGeneratedKeys();
            if(rs.next()) {
                id = rs.getLong(1);
                
                PreparedStatement stmtItemPedido = this.conexao.getConnection().prepareStatement(sqlItemPedido);                
                for (ItemPedido i : pedido.getItens()) {
                    stmtItemPedido.setLong(1, id);
                    stmtItemPedido.setLong(2, i.getProduto().getId());
                    stmtItemPedido.setLong(3, i.getFornecedor().getId());
                    stmtItemPedido.setDouble(4, i.getValorUnitario());
                    stmtItemPedido.setInt(5, i.getQuantidade());
                    stmtItemPedido.executeUpdate();                    
		}                
                
                this.conexao.commit();
            }            
        } catch(SQLException e) {
            this.conexao.rollback();
            System.out.println(e);
            throw e;
        }        
        return id;
    }

    public int alterar(Pedido pedido) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "UPDATE pedidos SET situacao = ? WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setString(1, pedido.getSituacao());
            stmt.setLong(2, pedido.getId());                        
            alterados = stmt.executeUpdate();
            this.conexao.commit(); 
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }

    public Pedido obterPedido(long id) throws SQLException, ClassNotFoundException {                        
        String sql = "SELECT p.id as id_pedido, p.date_time, p.situacao, u.id as id_usuario, u.nome as nome_usuario, u.email, e.*, prod.id as id_produto, prod.nome as nome_produto, ip.valor_unitario, ip.quantidade, f.nome as nome_fornecedor FROM pedidos as p" +
            "  INNER JOIN usuarios as u ON p.usuario_id = u.id" +
            "  INNER JOIN enderecos as e ON p.endereco_id = e.id" +
            "  INNER JOIN item_pedido as ip ON p.id = ip.pedido_id" +
            "  INNER JOIN produtos as prod ON ip.produto_id = prod.id" +
            "  INNER JOIN fornecedores as f ON ip.fornecedor_id = f.id" +
            "  WHERE p.id = ?";
        Pedido pedido = new Pedido();
        
        try {                                  
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setLong(1, id);
            
            ResultSet rs = stmt.executeQuery();
                
            Usuario usuario = new Usuario();
            Produto produto;
            ItemPedido itemPedido;
            Fornecedor fornecedor;
            Endereco endereco = new Endereco();
            List<ItemPedido> itensPedido = new ArrayList();
            
            //ler os dados de cada item do pedido
            while (rs.next()) {
                produto = new Produto();
                produto.setId(rs.getLong("id_produto"));
                produto.setNome(rs.getString("nome_produto"));
                
                fornecedor = new Fornecedor();
                fornecedor.setNome(rs.getString("nome_fornecedor"));
                        
                itemPedido = new ItemPedido();
                itemPedido.setProduto(produto);
                itemPedido.setFornecedor(fornecedor);
                itemPedido.setValorUnitario(rs.getDouble("valor_unitario"));
                itemPedido.setQuantidade(rs.getInt("quantidade"));
                
                itensPedido.add(itemPedido);                                                                
            }
            pedido.setItens(itensPedido);
            
            //volta a primeira linha para preencher os dados que são iguais em todas as linhas
            rs.first();
                        
            usuario.setId(rs.getLong("id_usuario"));
            usuario.setNome(rs.getString("nome_usuario"));
            usuario.setEmail(rs.getString("email"));

            endereco.setRua(rs.getString("rua"));
            endereco.setNumero(rs.getString("numero"));
            endereco.setCidade(rs.getString("cidade"));
            endereco.setCep(rs.getString("cep"));
            endereco.setComplemento(rs.getString("complemento"));

            pedido.setId(rs.getLong("id_pedido"));
            pedido.setDateTime(rs.getTimestamp("date_time"));
            pedido.setSituacao(rs.getString("p.situacao"));
            pedido.setEndereco(endereco);
            pedido.setUsuario(usuario);
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        
        return pedido;
    }
    
    public List<Pedido> listar() throws SQLException, ClassNotFoundException {                
        String sql = "SELECT p.*, u.*, e.*, SUM(ip.valor_unitario * ip.quantidade) as total"
                + " FROM pedidos as p"
                + " INNER JOIN usuarios as u ON p.usuario_id = u.id"
                + " INNER JOIN item_pedido as ip ON ip.pedido_id = p.id"
                + " INNER JOIN enderecos as e ON p.endereco_id = e.id"                
                + " GROUP BY p.id"
                + " ORDER BY p.date_time DESC;";     
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);                                    
            ResultSet rs = stmt.executeQuery();
            
            List<Pedido> pedidos = new ArrayList<>();
            
            while (rs.next()) {
                pedidos.add(parser(rs));
            }            
            
            return pedidos;
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }                       
    }
    
    public List<Pedido> listar(long idUsuario) throws SQLException, ClassNotFoundException {                
        String sql = "SELECT p.*, u.*,e.*, SUM(ip.valor_unitario * ip.quantidade) as total"
                + " FROM pedidos as p"
                + " INNER JOIN usuarios as u ON p.usuario_id = u.id"
                + " INNER JOIN item_pedido as ip ON p.id = ip.pedido_id"
                + " INNER JOIN enderecos as e ON p.endereco_id = e.id"
                + " WHERE u.id = ?"
                + " GROUP BY p.id"
                + " ORDER BY p.date_time DESC;";                             
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setLong(1, idUsuario);
            ResultSet rs = stmt.executeQuery();
            
            List<Pedido> pedidos = new ArrayList<>();
            
            while (rs.next()) {
                pedidos.add(parser(rs));
            }            
            
            return pedidos;
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }                       
    }
    
    public List<Pedido> listarDetalhado() throws SQLException, ClassNotFoundException {                        
        String sql = "SELECT p.id as id_pedido, p.date_time, p.situacao, u.id as id_usuario, u.telefone, u.nome as nome_usuario, u.email, e.*, prod.id as id_produto, prod.nome as nome_produto, ip.valor_unitario, ip.quantidade, f.nome as nome_fornecedor FROM pedidos as p" +
            "  INNER JOIN usuarios as u ON p.usuario_id = u.id" +
            "  INNER JOIN enderecos as e ON p.endereco_id = e.id" +
            "  INNER JOIN item_pedido as ip ON p.id = ip.pedido_id" +
            "  INNER JOIN produtos as prod ON ip.produto_id = prod.id" +
            "  INNER JOIN fornecedores as f ON ip.fornecedor_id = f.id" +
            "  ORDER BY p.date_time DESC;";
        List<Pedido> pedidos = new ArrayList<>();
        Pedido pedido = new Pedido();
        Usuario usuario;
        Produto produto;
        ItemPedido itemPedido;
        Fornecedor fornecedor;
        Endereco endereco;
        List<ItemPedido> itensPedido = new ArrayList();
        
        try {                                  
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            
            ResultSet rs = stmt.executeQuery();
            
            while (rs.next()) {            
                pedido = new Pedido();
                usuario = new Usuario();            
                endereco = new Endereco();
                itensPedido = new ArrayList();                        
            
                produto = new Produto();
                produto.setId(rs.getLong("id_produto"));
                produto.setNome(rs.getString("nome_produto"));
                
                fornecedor = new Fornecedor();
                fornecedor.setNome(rs.getString("nome_fornecedor"));
                        
                itemPedido = new ItemPedido();
                itemPedido.setProduto(produto);
                itemPedido.setFornecedor(fornecedor);
                itemPedido.setValorUnitario(rs.getDouble("valor_unitario"));
                itemPedido.setQuantidade(rs.getInt("quantidade"));                
                itensPedido.add(itemPedido);                                                        
                
                usuario.setId(rs.getLong("id_usuario"));
                usuario.setNome(rs.getString("nome_usuario"));
                usuario.setEmail(rs.getString("email"));
                usuario.setTelefone(rs.getString("telefone"));

                endereco.setRua(rs.getString("rua"));
                endereco.setNumero(rs.getString("numero"));
                endereco.setCidade(rs.getString("cidade"));
                endereco.setCep(rs.getString("cep"));
                endereco.setComplemento(rs.getString("complemento"));

                pedido.setItens(itensPedido);
                pedido.setId(rs.getLong("id_pedido"));
                pedido.setDateTime(rs.getTimestamp("date_time"));
                pedido.setSituacao(rs.getString("p.situacao"));
                pedido.setEndereco(endereco);
                pedido.setUsuario(usuario);
                
                pedidos.add(pedido);
            }                                                                        
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        
        return pedidos;
    }
    
    private Pedido parser(ResultSet rs) throws SQLException {
        Pedido p = new Pedido();        
        Usuario u = new Usuario();
        Endereco e = new Endereco();
        
        p.setId(rs.getLong("p.id"));
        p.setDateTime(rs.getTimestamp("p.date_time"));
        p.setSituacao(rs.getString("situacao"));
        p.setTotal(rs.getDouble("total"));
        u.setNome(rs.getString("u.nome"));
        u.setId(rs.getLong("u.id"));        
        u.setTelefone(rs.getString("u.telefone"));
        e.setBairro(rs.getString("bairro"));
        e.setRua(rs.getString("rua"));
        e.setNumero(rs.getString("numero"));
        e.setCidade(rs.getString("cidade"));
        e.setCep(rs.getString("cep"));
        
        p.setUsuario(u);
        p.setEndereco(e);
        
        return p;
    }        
}