package data;

import conexoes.ConexaoMysql;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class ProdutoDao {
    private final ConexaoMysql conexao;
    
    public ProdutoDao() throws SQLException, ClassNotFoundException {
        this.conexao = new ConexaoMysql();
    }
    
    public Long inserir(Produto produto) throws SQLException, ClassNotFoundException {
        Long id = null;
        String sql = "INSERT INTO produtos (nome, descricao, preco, ativo) VALUES (?, ?, ?, true);";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
            stmt.setString(1, produto.getNome());
            stmt.setString(2, produto.getDescricao());
            stmt.setFloat(3, produto.getPreco());
            stmt.executeUpdate();
            
            ResultSet rs = stmt.getGeneratedKeys();            
            if(rs.next()) {
                id = rs.getLong(1);
            }
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        
        return id;
    }
    
    public int alterar(Produto produto) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "UPDATE produtos SET nome = ?, descricao = ?, preco = ?, ativo = ? WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setString(1, produto.getNome());
            stmt.setString(2, produto.getDescricao());
            stmt.setFloat(3, produto.getPreco());
            stmt.setBoolean(4, produto.isAtivo());
            stmt.setLong(5, produto.getId());
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }
    
    public int excluir(long id) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "DELETE FROM produtos WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setLong(1, id);
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }
    
    public Produto obterProduto(long id) throws SQLException, ClassNotFoundException {                
        String sql = "SELECT p.*, "
                + " (SELECT SUM(ie.quantidade) FROM itens_estoque as ie WHERE produto_id = p.id) as quantidade "
                + " FROM produtos as p"
                + " WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setLong(1, id);
            
            ResultSet rs = stmt.executeQuery();
            if (rs.next()) {
                return parser(rs);
            }            
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        
        return null;
    }
    
    public List<Produto> listar() throws SQLException, ClassNotFoundException {                
        String sql = "SELECT p.*, "
                + " (SELECT SUM(ie.quantidade) FROM itens_estoque as ie WHERE produto_id = p.id) as quantidade"
                + " FROM produtos as p"
                + " WHERE p.ativo = true"
                + " ORDER BY id;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);                                    
            ResultSet rs = stmt.executeQuery();
            
            List<Produto> produtos = new ArrayList<>();
            
            while (rs.next()) {
                produtos.add(parser(rs));
            }            
            
            return produtos;
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }                       
    }
    
    private Produto parser(ResultSet rs) throws SQLException {
        Produto p = new Produto();
        
        p.setId(rs.getLong("id"));
        p.setNome(rs.getString("nome"));
        p.setDescricao(rs.getString("descricao"));
        p.setPreco(rs.getFloat("preco"));
        p.setQuantDisponivel(rs.getInt("quantidade"));
        p.setAtivo(rs.getBoolean("ativo"));
        
        return p;
    }
}
