package data;

import conexoes.ConexaoMysql;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class UsuarioDao {
    private final ConexaoMysql conexao;
    
    public UsuarioDao() throws SQLException, ClassNotFoundException {
        this.conexao = new ConexaoMysql();
    }
    
    public Long inserir(Usuario usuario, Endereco endereco) throws SQLException, ClassNotFoundException {
        Long idUsuario = null;        
        String sql = "INSERT INTO usuarios (nome, email, telefone, senha, is_admin, ativo) VALUES (?, ?, ?, ?, ?, true);";        
        String sqlEndereco = "INSERT INTO enderecos(cep, bairro, cidade, rua, numero, complemento, ativo) VALUES (?, ?, ?, ?, ?, ?, true);";        
        String sqlEnderecoUsuario = "INSERT INTO endereco_usuario(endereco_id, usuario_id) VALUES (?, ?);";

        try {                        
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql, Statement.RETURN_GENERATED_KEYS);
            stmt.setString(1, usuario.getNome());
            stmt.setString(2, usuario.getEmail());
            stmt.setString(3, usuario.getTelefone());
            stmt.setString(4, usuario.getSenha());
            stmt.setBoolean(5, usuario.isAdmin());
            stmt.executeUpdate();
            
            ResultSet rs = stmt.getGeneratedKeys();            
            if(rs.next()) {
                idUsuario = rs.getLong(1);
                
                PreparedStatement stmtEndereco = this.conexao.getConnection().prepareStatement(sqlEndereco, Statement.RETURN_GENERATED_KEYS);
                stmtEndereco.setString(1, endereco.getCep());
                stmtEndereco.setString(2, endereco.getBairro());
                stmtEndereco.setString(3, endereco.getCidade());
                stmtEndereco.setString(4, endereco.getRua());
                stmtEndereco.setString(5, endereco.getNumero());
                stmtEndereco.setString(6, endereco.getComplemento());                
                stmtEndereco.executeUpdate();
                
                rs = stmtEndereco.getGeneratedKeys();
                if(rs.next()) {
                    PreparedStatement stmtEnderecoUsuario = this.conexao.getConnection().prepareStatement(sqlEnderecoUsuario);
                    stmtEnderecoUsuario.setLong(1, rs.getLong(1));
                    stmtEnderecoUsuario.setLong(2, idUsuario);        
                    stmtEnderecoUsuario.executeUpdate();
                }
                
                this.conexao.commit();
            }            
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        
        return idUsuario;
    }

    public int alterar(Usuario usuario) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "UPDATE usuarios SET nome = ?, email = ?, senha = ?, telefone = ?, is_admin = ?, ativo = ? WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);
            stmt.setString(1, usuario.getNome());
            stmt.setString(2, usuario.getEmail());
            stmt.setString(3, usuario.getSenha());
            stmt.setString(4, usuario.getTelefone());
            stmt.setBoolean(5, usuario.isAdmin());
            stmt.setBoolean(6, usuario.isAtivo());
            stmt.setLong(7, usuario.getId());
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }

    public int excluir(long id) throws SQLException, ClassNotFoundException {        
        int alterados = 0;
        String sql = "DELETE FROM usuarios WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setLong(1, id);
            alterados = stmt.executeUpdate();
            this.conexao.commit();
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
        return alterados;
    }
    
    public Usuario obterUsuario(long id) throws SQLException, ClassNotFoundException {                
        String sql = "SELECT * FROM usuarios WHERE id = ?;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setLong(1, id);
            
            ResultSet rs = stmt.executeQuery();
            if (rs.next()) {
                return parser(rs);
            }            
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        
        return null;
    }
    
    public Usuario obterUsuarioComEmail(String email) throws SQLException, ClassNotFoundException {                
        String sql = "SELECT * FROM usuarios WHERE email = ?;";        
        Usuario u = null;
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);            
            stmt.setString(1, email);
            
            ResultSet rs = stmt.executeQuery();
            if (rs.next()) {
                u = parser(rs);
                u.setEnderecos(obterEnderecos(u));
            }
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }        
        
        return u;
    }
    
    public List<Usuario> listar() throws SQLException, ClassNotFoundException {                
        String sql = "SELECT id, nome, email, telefone, is_admin, ativo"
                + " FROM usuarios"
                + " WHERE ativo = true"
                + " ORDER BY id;";        
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);                                    
            ResultSet rs = stmt.executeQuery();
            
            List<Usuario> usuarios = new ArrayList<>();
            
            while (rs.next()) {
                usuarios.add(parser(rs));
            }            
            
            return usuarios;
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }                       
    }
    
    private Usuario parser(ResultSet rs) throws SQLException {
        Usuario u = new Usuario();
        
        u.setId(rs.getLong("id"));
        u.setNome(rs.getString("nome"));
        u.setEmail(rs.getString("email"));
        u.setTelefone(rs.getString("telefone"));
        u.setAtivo(rs.getBoolean("ativo"));
        try {
            u.setSenha(rs.getString("senha"));
        } catch (SQLException e) {
            u.setSenha("");
        }        
        u.setAdmin(rs.getBoolean("is_admin"));
        
        return u;
    }    

    private List<Endereco> obterEnderecos(Usuario u) throws SQLException {
        String sql = "SELECT e.id, e.cep, e.bairro, e.cidade, e.rua, e.numero, e.complemento" +
            " FROM enderecos as e" +
            " INNER JOIN endereco_usuario eu" +
            " WHERE (eu.usuario_id = ?) AND (eu.endereco_id = e.id);";
        
        try {
            PreparedStatement stmt = this.conexao.getConnection().prepareStatement(sql);                                    
            stmt.setLong(1, u.getId());
            ResultSet rs = stmt.executeQuery();
            
            List<Endereco> enderecos = new ArrayList<>();            

            while (rs.next()) {
                enderecos.add(parserEndereco(rs));
            }
            
            return enderecos;
        } catch(SQLException e) {
            this.conexao.rollback();
            throw e;
        }
    }

    private Endereco parserEndereco(ResultSet rs) throws SQLException {
        Endereco e = new Endereco();
        
        e.setId(rs.getLong("e.id"));
        e.setCep(rs.getString("cep"));
        e.setBairro(rs.getString("bairro"));
        e.setCidade(rs.getString("cidade"));
        e.setRua(rs.getString("rua"));
        e.setNumero(rs.getString("numero"));
        e.setComplemento(rs.getString("complemento"));        
        
        return e;
    }
}