package servicos;

import data.ItemEstoque;
import data.ItemEstoqueDao;
import data.Pedido;
import data.PedidoDao;
import data.Usuario;
import data.UsuarioDao;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;

@Path("admin/secure")
public class AdminService {
    @GET   
    @Produces(MediaType.APPLICATION_JSON)    
    @Path("/usuarios/")
    public Response list(@Context SecurityContext sc) {
        if (!sc.isUserInRole("admin")) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        try {            
            return Response.ok(new UsuarioDao().listar(), MediaType.APPLICATION_JSON).build();            
        } catch (ClassNotFoundException | SQLException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @DELETE
    @Path("/usuario/{id}/")
    public Response delete(@Context SecurityContext sc, @PathParam("id") long id) {                
        if (!sc.isUserInRole("admin")) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        if (((Usuario) sc.getUserPrincipal()).getId() == id) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        try {
            UsuarioDao udao = new UsuarioDao();
            Usuario u = udao.obterUsuario(id);                        
            
            u.setAtivo(false);
            udao.alterar(u);
            return Response.status(Response.Status.OK).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @PUT
    @Path("/usuario/")
    public Response update(@Context SecurityContext sc, Usuario usuario) {                
        if (!sc.isUserInRole("admin")) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        if (((Usuario) sc.getUserPrincipal()).getId() == usuario.getId()) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        try {
            UsuarioDao udao = new UsuarioDao();
            usuario.setSenha(udao.obterUsuario(usuario.getId()).getSenha());
            new UsuarioDao().alterar(usuario);
            return Response.status(Response.Status.OK).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @GET   
    @Produces(MediaType.APPLICATION_JSON)    
    @Path("/estoque/")
    public Response listItensEstoque(@Context SecurityContext sc) {
        if (!sc.isUserInRole("admin")) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        try {            
            return Response.ok(new ItemEstoqueDao().listar(), MediaType.APPLICATION_JSON).build();            
        } catch (ClassNotFoundException | SQLException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @POST
    @Consumes(MediaType.APPLICATION_JSON)    
    @Path("/estoque/")
    public Response addItemEstoque(@Context SecurityContext sc, ItemEstoque item) {           
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {            
            new ItemEstoqueDao().inserir(item);
            return Response.status(Response.Status.ACCEPTED).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/estoque/")
    public Response updateItemEstoque(@Context SecurityContext sc, ItemEstoque item) {           
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {            
            new ItemEstoqueDao().alterar(item);
            return Response.status(Response.Status.ACCEPTED).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @DELETE
    @Path("/estoque/")
    @Consumes(MediaType.APPLICATION_JSON)
    public Response deleteItemEstoque(@Context SecurityContext sc, ItemEstoque item) {                
        if (!sc.isUserInRole("admin")) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        try {
            new ItemEstoqueDao().excluir(item);
            return Response.status(Response.Status.OK).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @POST    
    @Path("/zerarEstoque/")
    public Response zerarEstoque(@Context SecurityContext sc, ItemEstoque item) {           
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {            
            new ItemEstoqueDao().zerarEstoque();
            return Response.status(Response.Status.ACCEPTED).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/pedidos/")
    public Response listarPedidos(@Context SecurityContext sc) throws IOException {        
        if (!sc.isUserInRole("admin")) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        try {                                   
            return Response.ok(new PedidoDao().listar(), MediaType.APPLICATION_JSON).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(AdminService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/pedido/{id}/")
    public Response getPedido(@Context SecurityContext sc, @PathParam("id") long id) {
        try {             
            if (!sc.isUserInRole("admin")) {
                return Response.status(Response.Status.UNAUTHORIZED).build();
            }
            Pedido p = new PedidoDao().obterPedido(id);            
            return Response.ok(p, MediaType.APPLICATION_JSON).build();            
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/pedido/")
    public Response updateSituacaoPedido(@Context SecurityContext sc, Pedido pedido) {           
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {             
            new PedidoDao().alterar(pedido);
            
            String email = "Olá <b>" + pedido.getUsuario().getNome() + "</b>,<br/>"
                    + "<p>O seu pedido de número " + pedido.getId() +  " foi alterado para a situação: <b>" + pedido.getSituacao() + "<b>";                                
            new EmailService().sendMail(pedido.getUsuario().getEmail(), "Notificação sobre seu pedido", email);
            
            return Response.status(Response.Status.ACCEPTED).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @GET
    @Path("/exportPedidos/")
    @Produces(MediaType.APPLICATION_OCTET_STREAM)
    public Response exportPedidos(@Context HttpServletResponse response){
        try{
            List<Pedido> list = new PedidoDao().listar();
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy HH:mm");

            response.setHeader("Content-Disposition", "attachment; filename=" + "pedidos.csv");        
            response.setContentType("text/csv");        
            OutputStreamWriter osw = new OutputStreamWriter(response.getOutputStream(), "UTF-8");

            CSVFormat csvFileFormat = CSVFormat.DEFAULT.withDelimiter(';').withRecordSeparator("\n");       
            CSVPrinter csvFilePrinter = new CSVPrinter(osw, csvFileFormat);

            List<String> row = new ArrayList<>();
            row.add("Número do pedido");
            row.add("Data");
            row.add("Situação");
            row.add("Valor total (R$)");
            row.add("Número do usuário");
            row.add("Usuário");
            row.add("Telefone");
            row.add("Endereço");
            row.add("Bairro");
            row.add("Cidade");
            row.add("CEP");
            
            csvFilePrinter.printRecord(row);

            for(Pedido pedido : list){
                row = new ArrayList<>();
                row.add(String.valueOf(pedido.getId()));
                row.add(String.valueOf(sdf.format(pedido.getDateTime())));
                row.add(pedido.getSituacao());                
                row.add(String.valueOf(new DecimalFormat("#.##").format(pedido.getTotal())));
                row.add(String.valueOf(pedido.getUsuario().getId()));
                row.add(pedido.getUsuario().getNome());
                row.add(pedido.getUsuario().getTelefone());                
                row.add(pedido.getEndereco().getRua() + ", " + pedido.getEndereco().getNumero());
                row.add(pedido.getEndereco().getBairro());
                row.add(pedido.getEndereco().getCidade());
                row.add(pedido.getEndereco().getCep());
                csvFilePrinter.printRecord(row);
            }

            csvFilePrinter.flush();
            csvFilePrinter.close();

            osw.flush();
            osw.close();

        } catch(IOException | ClassNotFoundException | SQLException e){
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }

        return Response.ok().build();
    }
    
    @GET
    @Path("/exportItensPedidos/")
    @Produces(MediaType.APPLICATION_OCTET_STREAM)
    public Response exportItensPedidos(@Context HttpServletResponse response){
        try{
            List<Pedido> list = new PedidoDao().listarDetalhado();            
            SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy HH:mm");

            response.setHeader("Content-Disposition", "attachment; filename=" + "pedidos.csv");        
            response.setContentType("text/csv");        
            OutputStreamWriter osw = new OutputStreamWriter(response.getOutputStream(), "UTF-8");

            CSVFormat csvFileFormat = CSVFormat.DEFAULT.withDelimiter(';').withRecordSeparator("\n");       
            CSVPrinter csvFilePrinter = new CSVPrinter(osw, csvFileFormat);

            List<String> row = new ArrayList<>();
            row.add("Número do pedido");
            row.add("Data");
            row.add("Situação");
            row.add("Número do cliente");
            row.add("Cliente");
            row.add("Telefone");
            row.add("Endereço");
            row.add("Produto");
            row.add("Preço unitário (R$)");
            row.add("Quantidade");
            row.add("Total (R$)");
            row.add("Fornecedor");                       
            
            csvFilePrinter.printRecord(row);

            for(Pedido pedido : list){                
                row = new ArrayList<>();
                row.add(String.valueOf(pedido.getId()));
                row.add(String.valueOf(sdf.format(pedido.getDateTime())));
                row.add(pedido.getSituacao());
                row.add(String.valueOf(pedido.getUsuario().getId()));
                row.add(pedido.getUsuario().getNome());
                row.add(pedido.getUsuario().getTelefone());
                row.add(pedido.getEndereco().getRua() + ", " + pedido.getEndereco().getNumero());
                row.add(String.valueOf(pedido.getItens().get(0).getProduto().getNome()));
                row.add(String.valueOf(new DecimalFormat("#.##").format(pedido.getItens().get(0).getValorUnitario())));
                row.add(String.valueOf(pedido.getItens().get(0).getQuantidade()));
                row.add(String.valueOf(new DecimalFormat("#.##").format(pedido.getItens().get(0).getValorUnitario() * pedido.getItens().get(0).getQuantidade())));
                row.add(String.valueOf(pedido.getItens().get(0).getFornecedor().getNome()));
                
                csvFilePrinter.printRecord(row);
            }

            csvFilePrinter.flush();
            csvFilePrinter.close();

            osw.flush();
            osw.close();

        } catch(IOException | ClassNotFoundException | SQLException e){
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }

        return Response.ok().build();
    }
}
