package servicos;

import data.Fornecedor;
import data.FornecedorDao;
import java.io.IOException;
import java.sql.SQLException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;

@Path("fornecedores")
public class FornecedorService {
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/secure/")
    public List<Fornecedor> list(@Context SecurityContext sc) throws IOException {
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {                       
            return new FornecedorDao().listar();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/secure/{id}/")
    public Fornecedor get(@Context SecurityContext sc, @PathParam("id") long id) {
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {            
            return new FornecedorDao().obterFornecedor(id);
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }    
    
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.TEXT_PLAIN)
    @Path("/secure/")
    public long add(@Context SecurityContext sc, Fornecedor fornecedor) {           
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {
            return new FornecedorDao().inserir(fornecedor);
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }        
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/secure/")
    public long update(@Context SecurityContext sc, Fornecedor fornecedor) {                
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {
            new FornecedorDao().alterar(fornecedor);
            return fornecedor.getId();            
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @DELETE
    @Path("/secure/{id}/")
    public Response delete(@Context SecurityContext sc, @PathParam("id") long id) {                
        if (!sc.isUserInRole("admin")) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        try {
            FornecedorDao fdao = new FornecedorDao();
            Fornecedor f = fdao.obterFornecedor(id);
            f.setAtivo(false);
            fdao.alterar(f);
            return Response.status(Response.Status.OK).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
}