package servicos;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTCreationException;
import data.Cadastro;
import data.Usuario;
import data.UsuarioDao;
import java.nio.charset.Charset;
import java.sql.SQLException;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import org.apache.commons.lang3.RandomStringUtils;
import org.mindrot.jbcrypt.BCrypt;

@Path("login")
public class LoginService {
    private final String senhaHmac = "secret";       
    
    @POST   
    @Produces(MediaType.APPLICATION_JSON)
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/")
    public Response autenticar(Usuario u) {
        try {                        
            Usuario usuario = new UsuarioDao().obterUsuarioComEmail(u.getEmail()); //obtem usuario do banco usando email
            if (usuario == null) {                                
                return Response.status(Response.Status.FORBIDDEN).build();
            }
            if (!usuario.isAtivo()) {
                return Response.status(Response.Status.FORBIDDEN).build();
            }
                       
            //compara senha fornecida com senha do banco usando BCrypt e lança exceção caso negativo            
            if (!BCrypt.checkpw(u.getSenha(), usuario.getSenha())) {
                return Response.status(Response.Status.FORBIDDEN).build();
            }
            
            usuario.setToken(gerarToken(u.getEmail())); //define token                        
            
            usuario.setSenha("");
            
            return Response.ok(usuario, MediaType.APPLICATION_JSON).build(); //retorna dados do usuario logado + token
        } catch (ClassNotFoundException | SQLException e) {            
            Logger.getLogger(LoginService.class.getName()).log(Level.SEVERE, null, e);
            return Response.status(Response.Status.FORBIDDEN).build();
        }
    }
    
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/resetSenha/")
    public Response resetSenha(String email) {
        Usuario u;
        UsuarioDao udao;
        
        try {
            udao = new UsuarioDao();
            u = udao.obterUsuarioComEmail(email);
            
            if (u == null) {
                return Response.status(Response.Status.FORBIDDEN).build();
            }                                    
            String novaSenha = RandomStringUtils.random(10, true, true);
            u.setSenha(BCrypt.hashpw(novaSenha, BCrypt.gensalt()));
            
            udao.alterar(u);
            
            new EmailService().sendMail(email, "Nova senha gerada", "Olá <b>" + u.getNome()
                    + "</b>,<br/>Sua nova senha para efetuar login é: " + novaSenha);
            
            return Response.status(Response.Status.ACCEPTED).build();
        } catch (SQLException | ClassNotFoundException ex) {
            Logger.getLogger(LoginService.class.getName()).log(Level.SEVERE, null, ex);
            return Response.status(Response.Status.INTERNAL_SERVER_ERROR).build();
        }                
    }
    
    @GET   
    @Produces(MediaType.APPLICATION_JSON)    
    @Path("/secure/")
    public Response verificarToken(@Context SecurityContext sc) {
        try {             
            Usuario u = (Usuario) sc.getUserPrincipal();
            u.setSenha("");
            return Response.ok(u, MediaType.APPLICATION_JSON).build(); //retorna dados do usuario logado                   
        } catch (Exception e) {            
            Logger.getLogger(LoginService.class.getName()).log(Level.SEVERE, null, e);
            return Response.status(Response.Status.FORBIDDEN).build();
        }
    }       
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/")
    public void put(Cadastro cadastro) {
        try {
            //hash no password antes de salvar no banco            
            cadastro.getUsuario().setSenha(BCrypt.hashpw(cadastro.getUsuario().getSenha(), BCrypt.gensalt()));            
            new UsuarioDao().inserir(cadastro.getUsuario(), cadastro.getEndereco());
        } catch (ClassNotFoundException | SQLException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }    
    
    private String gerarToken(String email) {
        try {            
            //gera algoritmo com HMAC256            
            Algorithm algorithm = Algorithm.HMAC256(senhaHmac);
                        
            String token = JWT.create()
                .withIssuer("backend")
                .withSubject(email)
                .withAudience("frontend")                
                .withExpiresAt(Date.from(ZonedDateTime.now().plusHours(720).toInstant()))
                .withIssuedAt(Date.from(ZonedDateTime.now().toInstant()))                    
                .sign(algorithm);
            return token;
        } catch (JWTCreationException e){
            Logger.getLogger(WebToken.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }        
    }
}
