package servicos;

import data.ItemEstoque;
import data.ItemEstoqueDao;
import data.ItemPedido;
import data.Pedido;
import data.PedidoDao;
import data.Produto;
import data.ProdutoDao;
import data.Usuario;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;

@Path("pedido")
public class PedidoService {
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/secure/")
    public Response inserir(@Context SecurityContext sc, Pedido pedido) {
        if (((Usuario) sc.getUserPrincipal()).getId() == pedido.getUsuario().getId()) {            
            salvarPedido(pedido);            
        } else {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        
        return Response.status(Response.Status.ACCEPTED).build();
    }        

    private Pedido salvarPedido(Pedido pedido) {        
        try {
            PedidoDao pedidoDao = new PedidoDao();
            ProdutoDao produtoDao = new ProdutoDao();
            ItemEstoqueDao ieDao = new ItemEstoqueDao();
            List<ItemEstoque> itensEstoque = ieDao.listar();
            List<ItemPedido> resumoPedido = new ArrayList();                        
            List<ItemPedido> novaLista = new ArrayList();            
            Produto produto;            
            int quantidade;                        
            
            forPedido: for (ItemPedido ip : pedido.getItens()) {                
                produto = produtoDao.obterProduto(ip.getId());
                ip.setProduto(produto);
                ip.setValorUnitario(produto.getPreco());
                
                quantidade = ip.getQuantidade();                                
                
                while (quantidade > 0) {
                    if (ip.getQuantidade() <= produto.getQuantDisponivel()) {                        
                        resumoPedido.add(ip);
                        for (ItemEstoque ie : itensEstoque) {
                            if (ie.getProduto().getId() == produto.getId()) {
                                if (ie.getQuantidade() != 0) {
                                    ieDao = new ItemEstoqueDao();
                                    if (quantidade <= ie.getQuantidade()) {                                                               
                                        ie.setQuantidade(ie.getQuantidade() - quantidade);                                
                                        ieDao.alterar(ie);   
                                        ip.setQuantidade(quantidade);
                                        ip.setFornecedor(ie.getFornecedor());                                          
                                        novaLista.add(new ItemPedido(ip.getProduto(), ip.getFornecedor(), ip.getPreco(), ip.getQuantidade(), ip.getId()));
                                        continue forPedido;
                                    } else {
                                        quantidade -= ie.getQuantidade();
                                        ip.setFornecedor(ie.getFornecedor());
                                        ip.setQuantidade(ie.getQuantidade());
                                        ie.setQuantidade(0);
                                        ieDao.alterar(ie);                                                                                                                                                                
                                        novaLista.add(new ItemPedido(ip.getProduto(), ip.getFornecedor(), ip.getPreco(), ip.getQuantidade(), ip.getId()));
                                    }
                                }                            
                            }                            
                        }                        
                    } else {
                        System.out.println("Erro na quantidade disponível de " + ip);
                        throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
                    }  
                }
            }    
            
            pedido.setItens(novaLista);
            pedido.setId(pedidoDao.inserir(pedido));
            
            String email = "Olá <b>" + pedido.getUsuario().getNome() + "</b>,<br/>"
                    + "<p>O número do pedido é: " + pedido.getId() +  ".<br/>"
                    + "Seu pedido foi realizado com sucesso, confira a lista dos produtos comprados:"
                    + "<br/>" + Pedido.listarItens(resumoPedido) + "<p>Endereço da entrega: "
                    + pedido.getEndereco().getRua() + ", " + pedido.getEndereco().getNumero()
                    + "<br/>Você será notificado quando seu pedido sair para entrega.</p>";
            
            new EmailService().sendMail(pedido.getUsuario().getEmail(), "Pedido realizado com sucesso!", email);
            return pedido;
        } catch (SQLException | ClassNotFoundException ex) {
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }                
    }            

}
