package servicos;

import data.Produto;
import data.ProdutoDao;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.SQLException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import org.glassfish.jersey.media.multipart.FormDataParam;

@Path("produtos")
public class ProdutoService {
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/")
    public List<Produto> list() throws IOException {
        try {           
            ProdutoDao produtoDao = new ProdutoDao();
            return produtoDao.listar();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }        
    
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("{id}/")
    public Produto get(@PathParam("id") long id) {
        try {
            ProdutoDao produtoDao = new ProdutoDao();
            return produtoDao.obterProduto(id);
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }    
    
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.TEXT_PLAIN)
    @Path("/secure/")
    public long add(@Context SecurityContext sc, Produto produto) {           
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {
            ProdutoDao produtoDao = new ProdutoDao();
            return produtoDao.inserir(produto);
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @POST
    @Consumes(MediaType.MULTIPART_FORM_DATA)
    @Path("/secure/upload/")
    public Response uploadFile(@FormDataParam("foto") InputStream uploadedInputStream, @FormDataParam("id") long id) throws IOException {
        String uploadedFileLocation = new File("../.").getCanonicalPath() + "/img/" + id + ".jpg";
        
        salvarArquivo(uploadedInputStream, uploadedFileLocation);
        String output = "File uploaded to : " + uploadedFileLocation;
        return Response.status(200).entity(output).build();
    }
    
    private void salvarArquivo(InputStream uploadedInputStream, String uploadedFileLocation) {
        try {
            OutputStream out;
            int read;
            byte[] bytes = new byte[1024];

            out = new FileOutputStream(new File(uploadedFileLocation));
            while ((read = uploadedInputStream.read(bytes)) != -1) {
                out.write(bytes, 0, read);
            }
            out.flush();
            out.close();
        } catch (IOException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/secure/")
    public long update(@Context SecurityContext sc, Produto produto) {                
        if (!sc.isUserInRole("admin")) {
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
        try {
            ProdutoDao produtoDao = new ProdutoDao();
            produtoDao.alterar(produto);
            return produto.getId();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @DELETE
    @Path("/secure/{id}/")
    public Response delete(@Context SecurityContext sc, @PathParam("id") long id) {                
        if (!sc.isUserInRole("admin")) {
            return Response.status(Response.Status.UNAUTHORIZED).build();
        }
        try {
            ProdutoDao produtoDao = new ProdutoDao();            
            Produto p = produtoDao.obterProduto(id);
            
            p.setAtivo(false);
            
            produtoDao.alterar(p);            
            return Response.status(Response.Status.OK).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
}
