package servicos;

import data.Endereco;
import data.EnderecoDao;
import data.Pedido;
import data.PedidoDao;
import data.Usuario;
import data.UsuarioDao;
import data.UsuarioSenhas;
import java.io.IOException;
import java.sql.SQLException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import org.mindrot.jbcrypt.BCrypt;

@Path("usuario")
public class UsuarioService {
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/secure/enderecos/")
    public List<Endereco> listEnderecos(@Context SecurityContext sc) throws IOException {               
        try {                       
            long id = ((Usuario) sc.getUserPrincipal()).getId();
            return new EnderecoDao().listar(id);
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(UsuarioService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.TEXT_PLAIN)
    @Path("/secure/enderecos/")
    public long addEndereco(@Context SecurityContext sc, Endereco endereco) {                   
        try {
            long idUsuario = ((Usuario) sc.getUserPrincipal()).getId();
            return new EnderecoDao().inserir(endereco, idUsuario);
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(UsuarioService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }        
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/secure/enderecos/")
    public int updateEndereco(@Context SecurityContext sc, Endereco endereco) {                
        try {            
            return new EnderecoDao().alterar(endereco);
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(UsuarioService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @DELETE
    @Path("/secure/enderecos/{id}/")
    public Response deleteEndereco(@Context SecurityContext sc, @PathParam("id") long id) {                        
        try {
            new EnderecoDao().excluir(id);
            return Response.status(Response.Status.OK).build();
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(UsuarioService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/secure/pedidos/")
    public List<Pedido> listarPedidos(@Context SecurityContext sc) throws IOException {                    
        try {                       
            long id = ((Usuario) sc.getUserPrincipal()).getId();
            return new PedidoDao().listar(id);
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(UsuarioService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @GET
    @Produces(MediaType.APPLICATION_JSON)
    @Path("/secure/pedido/{id}/")
    public Response getPedido(@Context SecurityContext sc, @PathParam("id") long id) {
        try {             
            Pedido p = new PedidoDao().obterPedido(id);
            if (((Usuario) sc.getUserPrincipal()).getId() == p.getUsuario().getId()) {
                return Response.ok(p, MediaType.APPLICATION_JSON).build();
            } else {
                return Response.status(Response.Status.FORBIDDEN).build();
            }
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/secure/pedido")
    public Response cancelarPedido(@Context SecurityContext sc, Pedido pedido) {                
        try {             
            Pedido p = new PedidoDao().obterPedido(pedido.getId());
            if (((Usuario) sc.getUserPrincipal()).getId() == p.getUsuario().getId()) {
                if (p.getSituacao().equals("Novo")) {
                    pedido.setSituacao("Cancelado");
                    new PedidoDao().alterar(pedido);
                    return Response.status(Response.Status.ACCEPTED).build();
                } else {
                    return Response.status(Response.Status.FORBIDDEN).build();
                }                                
            } else {
                return Response.status(Response.Status.FORBIDDEN).build();
            }
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }        
    }
    
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/secure/senha")
    public Response alterarSenha(@Context SecurityContext sc, UsuarioSenhas usuarioSenhas) {                
        try {                         
            if (((Usuario) sc.getUserPrincipal()).getId() == usuarioSenhas.getUsuario().getId()) {
                UsuarioDao udao = new UsuarioDao();
                Usuario u = udao.obterUsuario(usuarioSenhas.getUsuario().getId());
                if (BCrypt.checkpw(usuarioSenhas.getSenhaAtual(), u.getSenha())) {
                    u.setSenha(BCrypt.hashpw(usuarioSenhas.getSenhaNova(), BCrypt.gensalt()));
                    udao.alterar(u);
                    return Response.status(Response.Status.ACCEPTED).build();
                } else {
                    return Response.status(Response.Status.FORBIDDEN).build();
                }                                
            } else {
                return Response.status(Response.Status.FORBIDDEN).build();
            }
        } catch (SQLException | ClassNotFoundException e) {
            Logger.getLogger(ProdutoService.class.getName()).log(Level.SEVERE, null, e);
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }        
    }
}
