package servicos;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTVerificationException;
import com.auth0.jwt.interfaces.DecodedJWT;
import data.UsuarioDao;
import java.io.IOException;
import java.sql.SQLException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.Provider;
import security.ContextoDeSeguranca;

@Provider
public class WebToken implements ContainerRequestFilter {
    private final String senhaHmac = "secret";
    
    @Override
    public void filter(ContainerRequestContext requestContext) throws IOException {                
        if (requestContext.getUriInfo().getPath().contains("secure")) {            
            List<String> authHeader = requestContext.getHeaders().get("Authorization");            
            if (authHeader != null && authHeader.size() > 0) {
                String email;
                String authToken = authHeader.get(0);
                authToken = authToken.replaceFirst("Bearer ", "");
                
                email = verificarToken(authToken).getSubject();
                         
                //definir contexto de segurança
                String scheme = requestContext.getUriInfo().getRequestUri().getScheme();
                try {
                    requestContext.setSecurityContext(new ContextoDeSeguranca(new UsuarioDao().obterUsuarioComEmail(email), scheme));                    
                    return;
                } catch (SQLException | ClassNotFoundException ex) {
                    Logger.getLogger(WebToken.class.getName()).log(Level.SEVERE, null, ex);
                }
            }

            Response unauthorizedStatus = Response.status(Response.Status.UNAUTHORIZED).entity("User cannot access the resource.").build();
            requestContext.abortWith(unauthorizedStatus);
        }
    }    
    
    private DecodedJWT verificarToken(String token) {
        try {
            Algorithm algorithm = Algorithm.HMAC256(senhaHmac);
            JWTVerifier verifier = JWT.require(algorithm)
                .withIssuer("backend")
                .build();
            return verifier.verify(token);
        } catch (JWTVerificationException e){
            throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        }
    }    
}