/* global urlBase */

app = angular.module('app').controller('adminCtrl', function($http){
    var self = this;
    viacepUrl = 'https://viacep.com.br/ws/';
    
    self.iconesAdministrador = true;    //mostra icones da tela do administrador
    self.mostrarAlterarUsuarios = false;
    self.mostrarProduto = false;
    self.usuario = undefined;
    self.usuarios = undefined;
    self.produto = undefined;
    self.produtos = undefined;
    self.fornecedor = undefined;
    self.fornecedores = undefined;
    self.item = undefined;
    self.itens = undefined;
    self.pedido = undefined;
    self.pedidos = undefined;
    self.loadingPedido = false;
    
    function esconderViews () {
        self.mostrarUsuarios = false;
        self.mostrarProdutos = false;
        self.mostrarFornecedores = false;
        self.mostrarEstoque = false;
        self.mostrarPedidos = false;
        self.usuario = undefined;
        self.produto = undefined;
        self.fornecedor = undefined;
        self.item = undefined;
        self.pedido = undefined;
    };
    
    esconderViews();
    
    self.listarUsuarios = function() {
        if (!self.mostrarUsuarios) {
            $http({
                method: 'GET',
                url: urlBase + 'admin/secure/usuarios/',            
                headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
            }).then(function successCallback(response) {
                self.usuarios = response.data;
                esconderViews();
                self.mostrarUsuarios = true;                    
            }, function errorCallback(response) {
                console.log(response.status);
            });
        } else {
            self.mostrarUsuarios = false;
        }        
    };
    
    self.alterarUsuario = function(usuario) {
        self.usuario = usuario;
        self.mostrarUsuarios = false;
        self.iconesAdministrador = false;
    };
    
    self.salvarAlteracaoUsuario = function (usuario) {        
        $http({
            method: 'PUT',
            url: urlBase + 'admin/secure/usuario/',       
            data: usuario,
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.usuario = undefined;            
            self.listarUsuarios();
            self.iconesAdministrador = true;
        }, function errorCallback(response) {
            $('#tituloMensagem').text("Erro");
            $('#corpoMensagem').text("Não é possível alterar seu próprio usuário");        
            $('#mensagem').modal('show');
        }); 
    };
    
    self.cancelarAlteracaoUsuario = function () {        
        self.usuario = undefined;
        self.mostrarUsuarios = true;
        self.iconesAdministrador = true;
    };
    
    self.excluirUsuario = function(usuario) {
        $http({
            method: 'DELETE',
            url: urlBase + 'admin/secure/usuario/' + usuario.id + '/',
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.mostrarUsuarios = false;
            self.listarUsuarios();
        }, function errorCallback(response) {
            $('#tituloMensagem').text("Erro");
            $('#corpoMensagem').text("Não é possível excluir seu próprio usuário");        
            $('#mensagem').modal('show');
        });
    };
    
    self.listarProdutos = function() {
        if (!self.mostrarProdutos) {
            $http({
                method: 'GET',
                url: urlBase + 'produtos/'
            }).then(function successCallback(response) {
                self.produtos = response.data;
                esconderViews();
                self.mostrarProdutos = true;                
            }, function errorCallback(response) {
                console.log(response.status);
            });
        } else {
            self.mostrarProdutos = false;
        }        
    };
    self.novoProduto = function() {
        self.produto = {};
        self.mostrarProdutos = false;
    };
    self.salvarProduto = function() {
        if (self.produto.id) {
            metodo = 'PUT';
        } else {
            metodo = 'POST';
        }        
        self.produto.preco = self.produto.preco.toString().replace(",",".");        
        $http({
            method: metodo,
            url: urlBase + 'produtos/secure/',
            data: self.produto,
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.listarProdutos();
            self.produto = undefined;            
                        
            var foto = $('#inputFoto')[0].files[0];
            
            if (foto === undefined) return;
            
            var formData = new FormData();
            formData.append('id', response.data);
            formData.append('foto', foto);            
            
            $.ajax({
                type: 'POST',
                url: urlBase + 'produtos/secure/upload/',
                xhr: function() {  // Custom XMLHttpRequest
                    var myXhr = $.ajaxSettings.xhr();
                    return myXhr;
                },
                beforeSend: function(request) {
                    request.setRequestHeader("Authorization", 'Bearer ' + localStorage.getItem("token"));
                },
                success: function(data) {
                    console.log('Imagem salva: ' + data);
                },
                    data: formData,
                    cache: false,
                    contentType: false,
                    processData: false
            });  
        }, function errorCallback(response) {
            console.log(response.status);
        });
    };
    
    self.alterarProduto = function(produto) {
        self.produto = produto;
        self.mostrarProdutos = false;
    };
    
    self.cancelarAlteracaoProduto = function () {        
        self.produto = undefined;
        self.mostrarProdutos = true;
        self.iconesAdministrador = true;
    };
    
    self.excluirProduto = function(produto) {
        self.produto = produto;
        self.mostrarProdutos = false;

        $http({
            method: 'DELETE',
            url: urlBase + 'produtos/secure/' + self.produto.id + '/',
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.produto = undefined;
            self.listarProdutos();
        }, function errorCallback(response) {
            self.erro(response.status);
            console.log(response.status);
        });
    };
    
    self.buscaCep = function(cep) {
        if (cep !== undefined) {
            cep = cep.replace(/\D/g,"");
            if (cep.length === 8) {
                $http({
                    method: 'GET',
                    url: viacepUrl + cep + '/json/'
                }).then(function successCallback(response) {
                    if (response.data.erro) {
                        $('#tituloMensagem').text("Erro");
                        $('#corpoMensagem').text("CEP inválido");            
                        $('#mensagem').modal('show');
                    } else {                                 
                        self.fornecedor.endereco.bairro = response.data.bairro;
                        self.fornecedor.endereco.cidade = response.data.localidade;
                        self.fornecedor.endereco.rua = response.data.logradouro;
                    }
                }, function errorCallback(response) {
                    self.erro(response.status);
                });
            } else {
                $('#tituloMensagem').text("Erro");
                $('#corpoMensagem').text("CEP inválido");            
                $('#mensagem').modal('show');
            }
        }        
    };
    
    //-----------início das funções para manter fornecedores-----------
    self.listarFornecedores = function() {
        if (!self.mostrarFornecedores) {
            $http({
                method: 'GET',
                url: urlBase + 'fornecedores/secure/',
                headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
            }).then(function successCallback(response) {
                self.fornecedores = response.data;
                esconderViews();
                self.mostrarFornecedores = true;                    
            }, function errorCallback(response) {
                console.log(response.status);
            });
        } else {
            self.mostrarFornecedores = false;
        }        
    };
    
    self.novoFornecedor = function() {
        self.fornecedor = {};
        self.mostrarFornecedores = false;
    };

    self.alterarFornecedor = function(fornecedor) {
        self.fornecedor = fornecedor;
        self.mostrarFornecedores = false;
        self.iconesAdministrador = false;
    };
    
    self.salvarFornecedor = function (fornecedor) {
        if (self.fornecedor.id) {
            metodo = 'PUT';
        } else {
            metodo = 'POST';
        }
        
        $http({
            method: metodo,
            url: urlBase + 'fornecedores/secure/',       
            data: fornecedor,
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.fornecedor = undefined;            
            self.listarFornecedores();
            self.iconesAdministrador = true;
        }, function errorCallback(response) {
            console.log(response.status);
        }); 
    };
    
    self.cancelarAlteracaoFornecedor = function () {
        self.fornecedor = undefined;
        self.mostrarFornecedores = true;
        self.iconesAdministrador = true;
    };
    
    self.excluirFornecedor = function(fornecedor) {
        $http({
            method: 'DELETE',
            url: urlBase + 'fornecedores/secure/' + fornecedor.id + '/',
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.mostrarFornecedores = false;
            self.listarFornecedores();
        }, function errorCallback(response) {
            console.log(response.status);
        });
    };        
    //-----------fim das funções para manter fornecedores-----------
    
    //-----------início das funções para manter estoque-------------    
    self.listarEstoque = function() {
        if (!self.mostrarEstoque) {
            //listar produtos para seleções do estoque
            $http({
                method: 'GET',
                url: urlBase + 'produtos/'
            }).then(function successCallback(response) {
                self.produtos = response.data;        
            }, function errorCallback(response) {
                console.log(response.status);
            });
            //listar fornecedores para seleções do estoque
            $http({
                method: 'GET',
                url: urlBase + 'fornecedores/secure/',
                headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
            }).then(function successCallback(response) {
                self.fornecedores = response.data;                
            }, function errorCallback(response) {
                console.log(response.status);
            });
            //listar itens do estoque
            $http({
                method: 'GET',
                url: urlBase + 'admin/secure/estoque/',
                headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
            }).then(function successCallback(response) {
                self.estoque = response.data;
                esconderViews();
                self.mostrarEstoque = true;                    
            }, function errorCallback(response) {
                console.log(response.status);
            });
        } else {
            self.mostrarEstoque = false;
        }        
    };
    
    self.novoItemEstoque = function() {
        self.item = {};
        self.novoItem = true;
        self.mostrarEstoque = false;
    };

    self.alterarItemEstoque = function(item) {
        self.item = item;
        self.novoItem = false;
        self.mostrarEstoque = false;
        self.iconesAdministrador = false;
    };
    
    self.salvarItemEstoque = function (item) {
        if (!self.novoItem) {
            metodo = 'PUT';
        } else {
            metodo = 'POST';
        }

        $http({
            method: metodo,
            url: urlBase + 'admin/secure/estoque/',       
            data: item,
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.item = undefined;            
            self.listarEstoque();
            self.iconesAdministrador = true;
        }, function errorCallback(response) {
            console.log(response.status);
        });
    };
    
    self.cancelarAlteracaoItemEstoque = function () {
        self.item = undefined;
        self.mostrarEstoque = true;
        self.iconesAdministrador = true;
    };
    
    self.excluirItemEstoque = function(item) {        
        $http({
            method: 'DELETE',
            url: urlBase + 'admin/secure/estoque/',
            data: item,
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token"), "Content-Type": "application/json" }
        }).then(function successCallback(response) {
            self.mostrarEstoque = false;
            self.listarEstoque();
        }, function errorCallback(response) {
            console.log(response.status);
        });
    };
    
    self.showZerarEstoque = function() {
        $('#modalConfirmarZerarEstoque').modal('show');
    };
    
    self.zerarEstoque = function() {
        $('#modalConfirmarZerarEstoque').modal('hide');
        $http({
            method: 'POST',
            url: urlBase + 'admin/secure/zerarEstoque/',            
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.mostrarEstoque = false;
            self.listarEstoque();
        }, function errorCallback(response) {
            $('#tituloMensagem').text("Erro");
            $('#corpoMensagem').text("Não foi possível zerar o estoque: " + response.data);  
            $('#mensagem').modal('show');
        });
    };
    //-----------fim das funções para manter estoque-------------       
     
    //-----------início das funções para manter pedidos-------------    
    self.listarPedidos = function() {
        if (!self.mostrarPedidos) {            
            $http({
                method: 'GET',
                url: urlBase + 'admin/secure/pedidos/',
                headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token"), "Content-Type": "application/json" }
            }).then(function successCallback(response) {                
                self.pedidos = response.data;                
                esconderViews();
                self.mostrarPedidos = true;
            }, function errorCallback(response) {
                console.log(response.status);
            });            
        } else {
            self.mostrarPedidos = false;
        }        
    };
    
    self.mostrarPedido = function(p){
        $http({
            method: 'GET',
            url: urlBase + 'admin/secure/pedido/' + p.id + '/',            
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.pedido = response.data;
            self.mostrarPedidos = false;
            self.iconesAdministrador = false;                        
        }, function errorCallback(response) {
            console.log("Token inválido");
        });
    };
    
    self.cancelarAlteracaoPedido = function() {
        self.pedido = undefined;
        self.mostrarPedidos = true;
        self.iconesAdministrador = true;
    };
    
    self.salvarAlteracaoPedido = function(pedido) {
        self.loadingPedido = true;
        $http({
            method: 'PUT',
            url: urlBase + 'admin/secure/pedido/',       
            data: pedido,
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.pedido = undefined;            
            self.listarPedidos();
            self.iconesAdministrador = true;
            self.loadingPedido = false;
        }, function errorCallback(response) {
            $('#tituloMensagem').text("Erro");
            $('#corpoMensagem').text("Não foi possível salvar o pedido. Erro: " + response);        
            $('#mensagem').modal('show');
            self.loadingPedido = false;
        });
    };
    
    self.exportPedidos = function() {
        $.ajax({
            url: urlBase + 'admin/secure/exportPedidos/',
            method: 'GET',
            headers: {
                'Authorization': 'Bearer ' + localStorage.getItem("token")
            },
            xhrFields: {
                responseType: 'blob'
            },
            success: function (data) {
                var a = document.createElement('a');
                var url = window.URL.createObjectURL(data);
                a.href = url;
                a.download = 'extract.csv';
                a.click();
                window.URL.revokeObjectURL(url);
            }
        });
    };
    
    self.exportItens = function() {
        $.ajax({
            url: urlBase + 'admin/secure/exportItensPedidos/',
            method: 'GET',
            headers: {
                'Authorization': 'Bearer ' + localStorage.getItem("token")
            },
            xhrFields: {
                responseType: 'blob'
            },
            success: function (data) {
                var a = document.createElement('a');
                var url = window.URL.createObjectURL(data);
                a.href = url;
                a.download = 'extract.csv';
                a.click();
                window.URL.revokeObjectURL(url);
            }
        });
    };
});