/* global urlBase */

app = angular.module('app').controller('pedidoCtrl', function($location, $http, $rootScope, $scope){
    var self = this;    
    self.carrinho = [];
    self.total = 0;
    self.enderecos;
    self.showCarrinho = true;
    self.showConfirmacao = false;
    self.showLoading = false;
    self.pedidoRealizado = new Object();

    self.adicionarProduto = function(produto) {        
        var pos = undefined;
        var quantidade = undefined;
        for (var i = 0; i < self.carrinho.length; i++) {
            if (produto.id === self.carrinho[i].id) {                    
                pos = i;
            }
        }
        if (pos === undefined) {
            quantidade = produto.quantDisponivel;
        } else {
            quantidade = produto.quantDisponivel - self.carrinho[pos].quantidade;
        }        
        if (isNaN(produto.quantidade) || produto.quantidade === null) {
            $('#tituloMensagem').text("Erro");
            $('#corpoMensagem').text("Entre com a quantidade desejada");            
        } else if (produto.quantidade <= 0) {
            $('#tituloMensagem').text("Erro");
            $('#corpoMensagem').text("Entre com uma quantidade válida");
        } else if (produto.quantidade > quantidade) {
            $('#tituloMensagem').text("Erro");
            $('#corpoMensagem').text("Desculpe, só possuimos " + quantidade + " unidades de " + produto.nome + " em estoque");
        } else {                        
            if (pos === undefined) {
                self.carrinho.push(produto);
            } else {
                self.carrinho[pos].quantidade += produto.quantidade;
            }
            localStorage.setItem("carrinho", JSON.stringify(self.carrinho));
            self.calcularTotal();
            $('#tituloMensagem').text("Sucesso");
            $('#corpoMensagem').text(produto.quantidade + " unidades de " + produto.nome + " adicionado ao carrinho");
        }
        $('#mensagem').modal('show');               
    };
    
    self.removerProduto = function(produto) {
        var index = self.carrinho.indexOf(produto);
        self.carrinho.splice(index, 1);
        localStorage.setItem("carrinho", JSON.stringify(self.carrinho));
        self.calcularTotal();
    };
    
    self.calcularTotal = function() {
        var total = 0;
        angular.forEach(self.carrinho, function(produto) {
            total += produto.quantidade * produto.preco;
        });
        self.total = total;
    };
    
    self.confirmarPedido = function(usuario, endereco, carrinho) {        
        self.pedidoRealizado.usuario = usuario;
        self.pedidoRealizado.endereco = endereco;
        self.pedidoRealizado.itens = carrinho;               
                        
        self.showCarrinho = false;
        self.showLoading = true;
                    
        $http({
            method: 'POST',
            url: urlBase + 'pedido/secure/',       
            data: self.pedidoRealizado,
            headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
        }).then(function successCallback(response) {
            self.showLoading = false;
            self.showConfirmacao = true;            
            self.carrinho = [];
            self.total = 0;
            localStorage.removeItem("carrinho");
        }, function errorCallback(response) {
            self.showLoading = false;
            $('#tituloMensagem').text("Erro");
            $('#corpoMensagem').text("Erro ao finalizar pedido, por favor entrar em contato com redecampocidade@gmail.com");        
            $('#mensagem').modal('show');               
            console.log(response.status);
        });        
    };
    
    self.navegar = function(destino) {
        switch (destino) {
            case 'confirmarPedido':
                $location.path("/confirmar");
                break;
            case 'login':
                $location.path("/login");
                break;
            case 'cadastro':
                $location.path("/cadastro");
                break;
            default:
        }
    };
    
    self.activate = function (){                
        self.showConfirmacao = false;
        self.showCarrinho = true;
        //verifica se já existem itens no carrinho na storage do browser        
        if (localStorage.getItem("carrinho") !== null) {
            self.carrinho = JSON.parse(localStorage.getItem("carrinho"));
            self.calcularTotal();
        }
        if (localStorage.getItem("token") !== null) {
            $http({
                method: 'GET',
                url: urlBase + 'usuario/secure/enderecos/',
                headers: { 'Authorization': 'Bearer ' + localStorage.getItem("token") }
            }).then(function successCallback(response) {
                self.enderecos = response.data;
            }, function errorCallback(response) {
                console.log(response.status);
            });
        }        
    };        
    
    self.activate();
    
    $rootScope.$on('$routeChangeSuccess', function (event, current, previous, rejection) {
        if (current.$$route.originalPath === "/carrinho") {
            self.activate(); 
        }
    });
});